from pexpect import pxssh


class Node:
	def __init__(self, username, hostname, password, testCommand):
		self.username = username
		self.hostname = hostname
		self.password = password
		self.testCommand = testCommand


class NodeConnection(pxssh.pxssh):
	def __init__(self, node):
		pxssh.pxssh.__init__(self)
		self.node = node
		self.connected = False

	def __enter__(self):
		self.connect()
		return self

	def __exit__(self, exc_type, exc_val, exc_tb):
		self.disconnect()

	def connect(self):
		"""
		Connects to a node via ssh. This may take a few seconds (due to the underlying module).
		:return:
		"""
		try:
			self.login(self.node.hostname, self.node.username, self.node.password,
					   original_prompt="/home/" + self.node.username + ">")
		except pxssh.ExceptionPxssh:
			print("ERROR: failed to connect to node: " + self.node.ssh_id)
			raise
		self.connected = True
		print("Connected to: " + str(self.node.username) + "@" + str(self.node.hostname))

	def disconnect(self):
		if self.connected:
			self.logout()
			self.connected = False
			print("Disconnected from: " + str(self.node.username) + "@" + str(self.node.hostname))

	def prepare_dump_settings(self, sim_id):
		self.start_clt()
		# set sim id
		self.sendline("tx id " + str(sim_id))
		# set number of coefficients to dump
		self.sendline("rx ncoff 30")
		self.stop_clt()

	def start_transmissions(self, num_transmissions, transmission_delay):
		"""
		Non-Blocking, initiates a broadcast transmission on the node.
		:param num_transmissions: number of packets to broadcast
		:param transmission_delay: time delay between each consecutive transmission in seconds
				which must be in the range from 0.05 to 60
		"""
		self.start_clt()
		# the delay parameter is given in milliseconds
		self.sendline("tx delay " + str(int(transmission_delay * 1000)))
		self.clt_prompt()
		# this is non blocking
		self.sendline("tx " + str(num_transmissions))
		self.clt_prompt()
		self.stop_clt()

	def get_dump(self, sender, sim_id):
		rx_content = self.exec_cmd("cd /data/rx && ls -1")
		dump_files = list(
			filter(lambda filename: self.check_filename(filename, sender, sim_id), rx_content.split("\r\n")))
		if len(dump_files) == 0:
			raise FileNotFoundError("ERROR: Dump not found! (receiver: " + str(self.node.node_id) +
									", sender: " + str(sender.node_id) + ", sim_id: " + str(sim_id))
		if len(dump_files) > 1:
			raise Exception("ERROR: There are multiple dump files for receiver: " + str(self.node.node_id) +
							", sender: " + str(sender.node_id) + ", sim_id: " + str(sim_id) + "!")
		content = self.exec_cmd("cat " + dump_files[0])
		return Dump(self.node, sender, content)

	def clean_dump_dir(self):
		self.sendline("cd /data/rx && rm *")
		self.prompt()

	def start_clt(self):
		self.sendline("clt")
		self.clt_prompt()

	def stop_clt(self):
		self.sendline("quit")
		self.prompt()

	def clt_prompt(self):
		self.expect("CMD>")

	def check_filename(self, filename, sender, sim_id):
		# Dumped receive data is written to a file with name "dump-RR-CCC-TT-DDD.txt" where RR
		# identifies the receiver node, CCC is a run counter which increments when the daemon starts,
		# TT identifies the transmitting node, and DDD is a test identifier (sim_id)
		filename = filename.split(".txt")[0]
		split = filename.split("-")
		if len(split) < 5:
			return False
		fn_dump, fn_receiver, fn_run, fn_sender, fn_id = split
		return fn_dump == "dump" \
			   and int(fn_receiver) == self.node.node_id \
			   and int(fn_sender) == sender.node_id \
			   and int(fn_id) == sim_id

	def exec_cmd(self, cmd):
		"""
		:param cmd: command to execute on the shell
		:return: the output produced by the command as a string
		"""
		self.sendline(cmd)
		self.prompt()
		return self.before.decode("utf-8").split(cmd + "\r\r\n")[1]


if __name__ == "__main__":
	print("hello")